@@ -1,9 +1,12 @@
 /*  DreamChess
-**  Copyright (C) 2003-2005  The DreamChess project
 **
-**  This program is free software; you can redistribute it and/or modify
+**  DreamChess is the legal property of its developers, whose names are too
+**  numerous to list here. Please refer to the COPYRIGHT file distributed
+**  with this source distribution.
+**
+**  This program is free software: you can redistribute it and/or modify
 **  it under the terms of the GNU General Public License as published by
-**  the Free Software Foundation; either version 2 of the License, or
+**  the Free Software Foundation, either version 3 of the License, or
 **  (at your option) any later version.
 **
 **  This program is distributed in the hope that it will be useful,
@@ -12,8 +15,7 @@
 **  GNU General Public License for more details.
 **
 **  You should have received a copy of the GNU General Public License
-**  along with this program; if not, write to the Free Software
-**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+**  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #ifdef HAVE_CONFIG_H
@@ -37,6 +39,8 @@
 #include "dreamchess.h"
 #include "debug.h"
 #include "svn_version.h"
+#include "audio.h"
+#include "system_config.h"
 
 #ifdef HAVE_GETOPT_LONG
 #define OPTION_TEXT(L, S, T) "  " L "\t" S "\t" T "\n"
@@ -54,11 +58,19 @@ typedef struct move_list
 }
 move_list_t;
 
+typedef struct cl_options {
+	int width;
+	int height;
+	int fs;
+	char *engine;
+} cl_options_t;
+
 static ui_driver_t *ui;
 static config_t *config;
 static move_list_t san_list, fan_list, fullalg_list;
 static history_t *history;
 static int in_game;
+static int engine_error;
 
 static void move_list_play(move_list_t *list, char *move)
 {
@@ -188,6 +200,16 @@ int game_save( int slot )
     return retval;
 }
 
+void game_set_engine_error(int err)
+{
+    engine_error = err;
+}
+
+int game_get_engine_error()
+{
+    return engine_error;
+}
+
 static int do_move(move_t *move, int ui_update)
 {
     char *move_s, *move_f, *move_san;
@@ -347,18 +369,63 @@ void game_get_move_list(char ***list, int *total, int *view)
     *view = fan_list.view;
 }
 
-int use_ui_fullscreen=0;
+int set_resolution(int init)
+{
+    int width, height, fs, ms;
+    option_t *option = config_get_option("resolution");
+    config_resolution_t *res = option->selected->data;
 
-#ifdef __BEOS__
-int use_ui_width=480;
-int use_ui_height=360;
+    option = config_get_option("full_screen");
+    fs = option->selected->index;
+
+    option = config_get_option("multisampling");
+
+    ms = option->selected->index * 2;
+
+    if (res) {
+        width = res->w;
+        height = res->h;
+    }
+    else {
+        /* Custom */
+        option = config_get_option("custom_resolution_width");
+        width = option->value;
+        option = config_get_option("custom_resolution_height");
+        height = option->value;
+    }
+
+    if (init)
+        return ui->init(width, height, fs, ms);
+    else
+        return ui->resize(width, height, fs, ms);
+}
+
+static void init_resolution()
+{
+    if (set_resolution(1)) {
+        DBG_LOG("switching to failsafe video mode defaults");
+        config_set_failsafe_video();
+        config_save();
+
+        if (set_resolution(1)) {
+            exit(1);
+        }
+    }
+}
+
+void toggle_fullscreen()
+{
+#ifdef _WIN32
+    DBG_WARN("fullscreen toggling is currently not supported on win32");
 #else
-int use_ui_width=640;
-int use_ui_height=480;
+    option_t *option = config_get_option("full_screen");
+    option_select_value_by_index(option, 1 - option->selected->index);
+    set_resolution(0);
 #endif
+}
 
 #ifndef _arch_dreamcast
-static void parse_options(int argc, char **argv, ui_driver_t **ui_driver, char **engine)
+static void parse_options(int argc, char **argv, ui_driver_t **ui_driver, cl_options_t *cl_options)
 {
     int c;
 
@@ -380,12 +447,11 @@ static void parse_options(int argc, char **argv, ui_driver_t **ui_driver, char *
             {0, 0, 0, 0}
         };
 
-    while ((c = getopt_long(argc, argv, "1:fhlu:v:W:H:", options, &optindex)) > -1)
+    while ((c = getopt_long(argc, argv, "1:fhlu:v:W:H:", options, &optindex)) > -1) {
 #else
 
-    while ((c = getopt(argc, argv, "1:fhlu:v:W:H:")) > -1)
+    while ((c = getopt(argc, argv, "1:fhlu:v:W:H:")) > -1) {
 #endif /* HAVE_GETOPT_LONG */
-
         switch (c)
         {
         case 'h':
@@ -414,16 +480,16 @@ static void parse_options(int argc, char **argv, ui_driver_t **ui_driver, char *
             }
             break;
         case '1':
-            *engine = optarg;
+            cl_options->engine = optarg;
             break;
         case 'f':
-            use_ui_fullscreen=1;
+            cl_options->fs = 1;
             break;
         case 'W':
-            use_ui_width=atoi(optarg);
+            cl_options->width = atoi(optarg);
             break;
         case 'H':
-            use_ui_height=atoi(optarg);
+            cl_options->height = atoi(optarg);
             break;
         case 'v':
             {
@@ -442,12 +508,43 @@ static void parse_options(int argc, char **argv, ui_driver_t **ui_driver, char *
                 dbg_set_level(level);
             }
         }
+    }
 }
 #endif
 
+static void set_cl_options(cl_options_t *cl_options)
+{
+        option_t *option;
+
+	if (cl_options->engine) {
+	    option = config_get_option("1st_engine");
+	    option_string_set_text(option, cl_options->engine);
+	}
+
+	if (cl_options->fs) {
+	    option = config_get_option("full_screen");
+	    option_select_value_by_name(option, "On");
+	}
+
+	if (cl_options->width) {
+	    option = config_get_option("custom_resolution_width");
+	    option->value = cl_options->width;
+	    option = config_get_option("resolution");
+	    option_select_value_by_name(option, "Custom");
+        }
+
+        if (cl_options->height) {
+	    option = config_get_option("custom_resolution_height");
+	    option->value = cl_options->height;
+	    option = config_get_option("resolution");
+	    option_select_value_by_name(option, "Custom");
+        }
+}
+
 int dreamchess(void *data)
 {
-    char *engine = "dreamer";
+    cl_options_t cl_options = {};
+
 #ifndef _arch_dreamcast
 
     arguments_t *arg = data;
@@ -459,39 +556,52 @@ int dreamchess(void *data)
 
 #ifndef _arch_dreamcast
 
-    parse_options(arg->argc, arg->argv, &ui, &engine);
+    parse_options(arg->argc, arg->argv, &ui, &cl_options);
 #endif
 
+    config_init();
+
+    set_cl_options(&cl_options);
+
     if (!ui)
     {
         DBG_ERROR("failed to find a user interface driver");
         exit(1);
     }
 
-    comm_init(engine);
-    comm_send("xboard\n");
+    init_resolution();
 
-    ui->init(use_ui_width,use_ui_height,use_ui_fullscreen);
     while (1)
     {
         board_t board;
         int pgn_slot;
+        option_t *option;
 
         if (!(config = ui->config(&pgn_slot)))
             break;
 
+        ch_userdir();
+        option = config_get_option("1st_engine");
+
+        game_set_engine_error(comm_init(option->string));
+
+        comm_send("xboard\n");
+
         comm_send("new\n");
         comm_send("random\n");
 
         comm_send("sd %i\n", config->cpu_level);
         comm_send("depth %i\n", config->cpu_level);
 
+        if (config->difficulty == 0)
+	    comm_send("noquiesce\n");
+
         if (config->player[WHITE] == PLAYER_UI
-                && config->player[BLACK] == PLAYER_UI)
-            comm_send("force\n");
+	    && config->player[BLACK] == PLAYER_UI)
+	    comm_send("force\n");
 
         if (config->player[WHITE] == PLAYER_ENGINE)
-            comm_send("go\n");
+	    comm_send("go\n");
 
         in_game = 1;
         board_setup(&board);
@@ -530,6 +640,7 @@ int dreamchess(void *data)
                             engine_move = fullalg_to_move(&new_board, move_str);
                         if (engine_move)
                         {
+                            audio_play_sound(AUDIO_MOVE);
                             do_move(engine_move, 1);
                             free(engine_move);
                         }
@@ -580,13 +691,13 @@ int dreamchess(void *data)
             }
             ui->poll();
         }
+        comm_send("quit\n");
+        comm_exit();
         history_exit(history);
         move_list_exit(&san_list);
         move_list_exit(&fan_list);
         move_list_exit(&fullalg_list);
     }
-    comm_send("quit\n");
-    comm_exit();
     ui->exit();
     return 0;
 }
