@@ -1,9 +1,12 @@
 /*  DreamChess
-**  Copyright (C) 2004-2005  The DreamChess project
 **
-**  This program is free software; you can redistribute it and/or modify
+**  DreamChess is the legal property of its developers, whose names are too
+**  numerous to list here. Please refer to the COPYRIGHT file distributed
+**  with this source distribution.
+**
+**  This program is free software: you can redistribute it and/or modify
 **  it under the terms of the GNU General Public License as published by
-**  the Free Software Foundation; either version 2 of the License, or
+**  the Free Software Foundation, either version 3 of the License, or
 **  (at your option) any later version.
 **
 **  This program is distributed in the hope that it will be useful,
@@ -12,8 +15,7 @@
 **  GNU General Public License for more details.
 **
 **  You should have received a copy of the GNU General Public License
-**  along with this program; if not, write to the Free Software
-**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+**  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #ifndef DREAMCHESS_H
@@ -35,6 +37,7 @@ typedef struct config
 {
     int player[2];
     int cpu_level;
+    int difficulty;
 }
 config_t;
 
@@ -52,5 +55,10 @@ void game_get_move_list(char ***list, int *total, int *view);
 int game_save( int slot );
 int game_load( int slot );
 void game_make_move_str(char *move_str, int ui_update);
+int game_get_engine_error();
+void game_set_engine_error(int err);
+
+int set_resolution(int init);
+void toggle_fullscreen();
 
 #endif /* DREAMCHESS_H */
