@@ -1,4 +1,4 @@
-/* A Bison parser, made by GNU Bison 2.2.  */
+/* A Bison parser, made by GNU Bison 2.3.  */
 
 /* Skeleton implementation for Bison's Yacc-like parsers in C
 
@@ -47,7 +47,7 @@
 #define YYBISON 1
 
 /* Bison version.  */
-#define YYBISON_VERSION "2.2"
+#define YYBISON_VERSION "2.3"
 
 /* Skeleton name.  */
 #define YYSKELETON_NAME "yacc.c"
@@ -86,7 +86,7 @@
 
 
 /* Copy the first part of user declarations.  */
-#line 22 "san_parse.y"
+#line 24 "san_parse.y"
 
 #include <stdlib.h>
 #include <string.h>
@@ -422,10 +422,10 @@ static const yytype_int8 yyrhs[] =
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const yytype_uint8 yyrline[] =
 {
-       0,    36,    36,    39,    43,    45,    47,    49,    51,    53,
-      55,    57,    59,    62,    63,    64,    65,    66,    69,    70,
-      73,    74,    77,    80,    83,    86,    89,    90,    91,    94,
-      95,    96,    97
+       0,    38,    38,    41,    45,    47,    49,    51,    53,    55,
+      57,    59,    61,    64,    65,    66,    67,    68,    71,    72,
+      75,    76,    79,    82,    85,    88,    91,    92,    93,    96,
+      97,    98,    99
 };
 #endif
 
@@ -665,13 +665,13 @@ do {									  \
 #if (defined __STDC__ || defined __C99__FUNC__ \
      || defined __cplusplus || defined _MSC_VER)
 static void
-yy_symbol_value_print (FILE *yyoutput, int yytype, const YYSTYPE * const yyvaluep)
+yy_symbol_value_print (FILE *yyoutput, int yytype, YYSTYPE const * const yyvaluep)
 #else
 static void
 yy_symbol_value_print (yyoutput, yytype, yyvaluep)
     FILE *yyoutput;
     int yytype;
-    const YYSTYPE * const yyvaluep;
+    YYSTYPE const * const yyvaluep;
 #endif
 {
   if (!yyvaluep)
@@ -697,13 +697,13 @@ yy_symbol_value_print (yyoutput, yytype, yyvaluep)
 #if (defined __STDC__ || defined __C99__FUNC__ \
      || defined __cplusplus || defined _MSC_VER)
 static void
-yy_symbol_print (FILE *yyoutput, int yytype, const YYSTYPE * const yyvaluep)
+yy_symbol_print (FILE *yyoutput, int yytype, YYSTYPE const * const yyvaluep)
 #else
 static void
 yy_symbol_print (yyoutput, yytype, yyvaluep)
     FILE *yyoutput;
     int yytype;
-    const YYSTYPE * const yyvaluep;
+    YYSTYPE const * const yyvaluep;
 #endif
 {
   if (yytype < YYNTOKENS)
@@ -751,15 +751,12 @@ do {								\
 #if (defined __STDC__ || defined __C99__FUNC__ \
      || defined __cplusplus || defined _MSC_VER)
 static void
-yy_reduce_print (YYSTYPE *yyvsp, 
-		   int yyrule)
+yy_reduce_print (YYSTYPE *yyvsp, int yyrule)
 #else
 static void
-yy_reduce_print (yyvsp, yyrule
-		   )
+yy_reduce_print (yyvsp, yyrule)
     YYSTYPE *yyvsp;
-    
-		   int yyrule;
+    int yyrule;
 #endif
 {
   int yynrhs = yyr2[yyrule];
@@ -1356,93 +1353,93 @@ yyreduce:
   switch (yyn)
     {
         case 4:
-#line 43 "san_parse.y"
+#line 45 "san_parse.y"
     { san_move.type = SAN_QUEENSIDE_CASTLE; }
     break;
 
   case 5:
-#line 45 "san_parse.y"
+#line 47 "san_parse.y"
     { san_move.type = SAN_KINGSIDE_CASTLE; }
     break;
 
   case 13:
-#line 62 "san_parse.y"
+#line 64 "san_parse.y"
     { san_move.piece = SAN_QUEEN; }
     break;
 
   case 14:
-#line 63 "san_parse.y"
+#line 65 "san_parse.y"
     { san_move.piece = SAN_ROOK; }
     break;
 
   case 15:
-#line 64 "san_parse.y"
+#line 66 "san_parse.y"
     { san_move.piece = SAN_BISHOP; }
     break;
 
   case 16:
-#line 65 "san_parse.y"
+#line 67 "san_parse.y"
     { san_move.piece = SAN_KNIGHT; }
     break;
 
   case 17:
-#line 66 "san_parse.y"
+#line 68 "san_parse.y"
     { san_move.piece = SAN_KING; }
     break;
 
   case 22:
-#line 77 "san_parse.y"
+#line 79 "san_parse.y"
     { san_move.source_file = (yyvsp[(1) - (1)]); }
     break;
 
   case 23:
-#line 80 "san_parse.y"
+#line 82 "san_parse.y"
     { san_move.source_rank = (yyvsp[(1) - (1)]); }
     break;
 
   case 24:
-#line 83 "san_parse.y"
+#line 85 "san_parse.y"
     { san_move.type = SAN_CAPTURE; }
     break;
 
   case 25:
-#line 86 "san_parse.y"
+#line 88 "san_parse.y"
     { san_move.destination = 8 * (yyvsp[(1) - (2)]) + (yyvsp[(2) - (2)]); }
     break;
 
   case 27:
-#line 90 "san_parse.y"
+#line 92 "san_parse.y"
     { san_move.state = SAN_STATE_CHECK; }
     break;
 
   case 28:
-#line 91 "san_parse.y"
+#line 93 "san_parse.y"
     { san_move.state = SAN_STATE_CHECKMATE; }
     break;
 
   case 29:
-#line 94 "san_parse.y"
+#line 96 "san_parse.y"
     { san_move.promotion_piece = SAN_QUEEN; }
     break;
 
   case 30:
-#line 95 "san_parse.y"
+#line 97 "san_parse.y"
     { san_move.promotion_piece = SAN_ROOK; }
     break;
 
   case 31:
-#line 96 "san_parse.y"
+#line 98 "san_parse.y"
     { san_move.promotion_piece = SAN_BISHOP; }
     break;
 
   case 32:
-#line 97 "san_parse.y"
+#line 99 "san_parse.y"
     { san_move.promotion_piece = SAN_KNIGHT; }
     break;
 
 
 /* Line 1267 of yacc.c.  */
-#line 1446 "san_parse.c"
+#line 1443 "san_parse.c"
       default: break;
     }
   YY_SYMBOL_PRINT ("-> $$ =", yyr1[yyn], &yyval, &yyloc);
@@ -1651,11 +1648,12 @@ yyreturn:
   if (yymsg != yymsgbuf)
     YYSTACK_FREE (yymsg);
 #endif
-  return yyresult;
+  /* Make sure YYID is used.  */
+  return YYID (yyresult);
 }
 
 
-#line 99 "san_parse.y"
+#line 101 "san_parse.y"
 
 
 #include <stdio.h>
