@@ -1,4 +1,4 @@
-/* A Bison parser, made by GNU Bison 2.2.  */
+/* A Bison parser, made by GNU Bison 2.3.  */
 
 /* Skeleton implementation for Bison's Yacc-like parsers in C
 
@@ -47,7 +47,7 @@
 #define YYBISON 1
 
 /* Bison version.  */
-#define YYBISON_VERSION "2.2"
+#define YYBISON_VERSION "2.3"
 
 /* Skeleton name.  */
 #define YYSKELETON_NAME "yacc.c"
@@ -106,12 +106,12 @@
 
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
-#line 19 "pgn_parser.y"
+#line 21 "pgn_parser.y"
 {
     int yyint;
     char *yycharp;
 }
-/* Line 193 of yacc.c.  */
+/* Line 187 of yacc.c.  */
 #line 116 "pgn_parser.c"
 	YYSTYPE;
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
@@ -122,7 +122,7 @@ typedef union YYSTYPE
 
 
 /* Copy the second part of user declarations.  */
-#line 30 "pgn_parser.y"
+#line 32 "pgn_parser.y"
 
 #include "dreamchess.h"
 
@@ -422,9 +422,9 @@ static const yytype_int8 yyrhs[] =
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const yytype_uint8 yyrline[] =
 {
-       0,    41,    41,    44,    47,    48,    51,    54,    57,    60,
-      63,    64,    65,    68,    69,    70,    73,    76,    79,    82,
-      85,    86,    89,    92
+       0,    43,    43,    46,    49,    50,    53,    56,    59,    62,
+      65,    66,    67,    70,    71,    72,    75,    78,    81,    84,
+      87,    88,    91,    94
 };
 #endif
 
@@ -657,13 +657,13 @@ do {									  \
 #if (defined __STDC__ || defined __C99__FUNC__ \
      || defined __cplusplus || defined _MSC_VER)
 static void
-yy_symbol_value_print (FILE *yyoutput, int yytype, const YYSTYPE * const yyvaluep)
+yy_symbol_value_print (FILE *yyoutput, int yytype, YYSTYPE const * const yyvaluep)
 #else
 static void
 yy_symbol_value_print (yyoutput, yytype, yyvaluep)
     FILE *yyoutput;
     int yytype;
-    const YYSTYPE * const yyvaluep;
+    YYSTYPE const * const yyvaluep;
 #endif
 {
   if (!yyvaluep)
@@ -689,13 +689,13 @@ yy_symbol_value_print (yyoutput, yytype, yyvaluep)
 #if (defined __STDC__ || defined __C99__FUNC__ \
      || defined __cplusplus || defined _MSC_VER)
 static void
-yy_symbol_print (FILE *yyoutput, int yytype, const YYSTYPE * const yyvaluep)
+yy_symbol_print (FILE *yyoutput, int yytype, YYSTYPE const * const yyvaluep)
 #else
 static void
 yy_symbol_print (yyoutput, yytype, yyvaluep)
     FILE *yyoutput;
     int yytype;
-    const YYSTYPE * const yyvaluep;
+    YYSTYPE const * const yyvaluep;
 #endif
 {
   if (yytype < YYNTOKENS)
@@ -743,15 +743,12 @@ do {								\
 #if (defined __STDC__ || defined __C99__FUNC__ \
      || defined __cplusplus || defined _MSC_VER)
 static void
-yy_reduce_print (YYSTYPE *yyvsp, 
-		   int yyrule)
+yy_reduce_print (YYSTYPE *yyvsp, int yyrule)
 #else
 static void
-yy_reduce_print (yyvsp, yyrule
-		   )
+yy_reduce_print (yyvsp, yyrule)
     YYSTYPE *yyvsp;
-    
-		   int yyrule;
+    int yyrule;
 #endif
 {
   int yynrhs = yyr2[yyrule];
@@ -1348,23 +1345,23 @@ yyreduce:
   switch (yyn)
     {
         case 7:
-#line 54 "pgn_parser.y"
+#line 56 "pgn_parser.y"
     {/*printf("Tag name: %s\n", yylval.yycharp);*/}
     break;
 
   case 8:
-#line 57 "pgn_parser.y"
+#line 59 "pgn_parser.y"
     {/*printf("Tag value: %s\n", yylval.yycharp);*/}
     break;
 
   case 18:
-#line 79 "pgn_parser.y"
+#line 81 "pgn_parser.y"
     {game_make_move_str(yylval.yycharp, 0);/*printf("Found move: %s\n", yylval.yycharp);*/}
     break;
 
 
 /* Line 1267 of yacc.c.  */
-#line 1368 "pgn_parser.c"
+#line 1365 "pgn_parser.c"
       default: break;
     }
   YY_SYMBOL_PRINT ("-> $$ =", yyr1[yyn], &yyval, &yyloc);
@@ -1573,11 +1570,12 @@ yyreturn:
   if (yymsg != yymsgbuf)
     YYSTACK_FREE (yymsg);
 #endif
-  return yyresult;
+  /* Make sure YYID is used.  */
+  return YYID (yyresult);
 }
 
 
-#line 95 "pgn_parser.y"
+#line 97 "pgn_parser.y"
 
 
 int yyerror(const char *s)
