@@ -1,12 +1,39 @@
+/*  DreamChess
+**
+**  DreamChess is the legal property of its developers, whose names are too
+**  numerous to list here. Please refer to the COPYRIGHT file distributed
+**  with this source distribution.
+**
+**  This program is free software: you can redistribute it and/or modify
+**  it under the terms of the GNU General Public License as published by
+**  the Free Software Foundation, either version 3 of the License, or
+**  (at your option) any later version.
+**
+**  This program is distributed in the hope that it will be useful,
+**  but WITHOUT ANY WARRANTY; without even the implied warranty of
+**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+**  GNU General Public License for more details.
+**
+**  You should have received a copy of the GNU General Public License
+**  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*/
 
 #include "ui_sdlgl.h"
 #include "svn_version.h"
+#include "audio.h"
+#include "options.h"
+#include "theme.h"
+#include "system_config.h"
+#include "gamegui_dialogs.h"
 
 static gg_dialog_style_t style_ingame, style_menu;
 static int turn_counter_start=0;
 static texture_t menu_title_tex;
+static int game_in_stalemate;
 static int white_in_check;
 static int black_in_check;
+static int white_in_checkmate;
+static int black_in_checkmate;
 static board_t board;
 static int pgn_slot;
 static int quit_to_menu=FALSE;
@@ -14,17 +41,20 @@ static int title_process_retval;
 static int set_loading=FALSE;
 static int dialog_promote_piece;
 static SDL_Joystick *joy;
-static char** themelist;
-static char** stylelist;
-static int num_style;
-static int pieces_list_total;
-static char** pieces_list;
-static char** board_list;
-static int board_list_total;
 static int show_egg;
 static int egg_req1=FALSE;
 static int screen_width=640;
 static int screen_height=480;
+static int screen_fs=0;
+static int screen_ms=0;
+static int reflections=1;
+static int mode_set_failed=0;
+static int engine_error_shown;
+
+static void music_callback(char *title, char *artist, char *album)
+{
+    DBG_LOG("now playing: %s - %s", artist, title);
+}
 
 int get_screen_width()
 {
@@ -36,15 +66,9 @@ int get_screen_height()
     return screen_height;
 }
 
-void toggle_fullscreen()
-{
-    SDL_WM_ToggleFullScreen( SDL_GetVideoSurface() );
-}
-
 static int menu_state;
 enum {
     MENU_STATE_FADE_IN,
-    MENU_STATE_PRESS_KEY,
     MENU_STATE_IN_MENU,
     MENU_STATE_LOAD,
     MENU_STATE_FADE_OUT,
@@ -68,41 +92,6 @@ int get_show_egg()
     return show_egg;
 }
 
-int get_pieces_list_total()
-{
-    return pieces_list_total;
-}
-
-int get_board_list_total()
-{
-    return board_list_total;
-}
-
-int get_num_style()
-{
-    return num_style;
-}
-
-char *get_board_list( int index )
-{
-    return board_list[index];
-}
-
-char *get_pieces_list( int index )
-{
-    return pieces_list[index];
-}
-
-char *get_stylelist( int index )
-{
-    return stylelist[index];
-}
-
-char *get_themelist( int index )
-{
-    return themelist[index];
-}
-
 int get_dialog_promote_piece()
 {
     return dialog_promote_piece;
@@ -163,6 +152,11 @@ void set_fading_out(int fade)
     fading_out=fade;
 }
 
+int get_game_stalemate()
+{
+    return game_in_stalemate;
+}
+
 int get_white_in_check()
 {
     return white_in_check;
@@ -173,6 +167,16 @@ int get_black_in_check()
     return black_in_check;
 }
 
+int get_white_in_checkmate()
+{
+    return white_in_checkmate;
+}
+
+int get_black_in_checkmate()
+{
+    return black_in_checkmate;
+}
+
 gg_dialog_style_t *get_ingame_style()
 {
     return &style_ingame;
@@ -198,6 +202,8 @@ static int poll_event(gg_event_t *event)
     gg_event_t gg_event;
     SDL_Event sdl_event;
 
+    audio_poll(1);
+
     while (SDL_PollEvent(&sdl_event))
     {
 
@@ -205,8 +211,9 @@ static int poll_event(gg_event_t *event)
             /* FIXME */
             exit(0);
 
-        if ( sdl_event.type == SDL_KEYDOWN && sdl_event.key.keysym.mod & KMOD_ALT &&
-            sdl_event.key.keysym.sym == SDLK_RETURN )
+        if ( (sdl_event.type == SDL_KEYDOWN && sdl_event.key.keysym.mod & KMOD_ALT &&
+            sdl_event.key.keysym.sym == SDLK_RETURN) || (sdl_event.type == SDL_KEYDOWN &&
+            sdl_event.key.keysym.sym == SDLK_F11) )
         {
             DBG_LOG( "toggled fullscreen" );
             toggle_fullscreen();
@@ -228,28 +235,28 @@ static int poll_event(gg_event_t *event)
     return 0;
 }
 
-static void draw_press_key_message()
-{
-    char *msg = "Press any key or button to start";
-
-    text_draw_string_bouncy(SCREEN_WIDTH / 2 - text_width(msg) * 0.75, 40,
-                            msg, 1.5f, get_col(COL_WHITE));
-}
-
 /** Implements ui_driver::menu */
 static config_t *do_menu(int *pgn)
 {
+#ifdef _arch_dreamcast
     SDL_Joystick *joy1=SDL_JoystickOpen(0);
+#endif
     title_process_retval=2;
 
     resize_window(screen_width, screen_height);
     glEnable(GL_BLEND);
     glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 
+    engine_error_shown = 0;
     set_loading=FALSE;
     draw_credits(1);
     open_title_root_dialog();
 
+    if (mode_set_failed) {
+        gg_dialog_open(dialog_error_create(gg_dialog_get_active(), "Error: failed to set video mode; using defaults", NULL));
+        mode_set_failed = 0;
+    }
+
     set_fade_start(gg_system_get_ticks());
     set_show_egg(FALSE);
 
@@ -262,18 +269,20 @@ static config_t *do_menu(int *pgn)
     while ( 1 )
     {
         Uint8 *keystate;
-        int mouse_x, mouse_y;
         gg_event_t event;
 
         keystate = SDL_GetKeyState(NULL);
 
         egg_req1=FALSE;
 
-        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
         gg_dialog_cleanup();
 
         /* Draw the menu.. */
+        glDisable(GL_BLEND);
+        glDisable(GL_DEPTH_TEST);
         draw_texture(&menu_title_tex, 0, 0, 640, 480, 1.0f, get_col(COL_WHITE));
+        glEnable(GL_BLEND);
+        glEnable(GL_DEPTH_TEST);
         text_draw_string_right(620, 20, "v" PACKAGE_VERSION " (r" SVN_VERSION ")", 0.75f, get_col(COL_WHITE));
 
         /*if (get_show_egg())
@@ -283,28 +292,15 @@ static config_t *do_menu(int *pgn)
         {
         case MENU_STATE_FADE_IN:
             while (poll_event(&event));
-            draw_press_key_message();
+            gg_dialog_render_all();
 
             if (!draw_fade(FADE_IN))
             {
-                menu_state = MENU_STATE_PRESS_KEY;
+                menu_state = MENU_STATE_IN_MENU;
                 draw_credits(1);
             }
             break;
 
-        case MENU_STATE_PRESS_KEY:
-            draw_press_key_message();
-            draw_credits(0);
-
-            while (poll_event(&event))
-            {
-                if (event.type == GG_EVENT_KEY
-                        || (event.type == GG_EVENT_MOUSE
-                            && event.mouse.type == GG_MOUSE_BUTTON_DOWN))
-                    menu_state = MENU_STATE_IN_MENU;
-            }
-            break;
-
         case MENU_STATE_IN_MENU:
 
             if ( keystate[SDLK_UP] )
@@ -336,22 +332,13 @@ static config_t *do_menu(int *pgn)
             break;
 
         case MENU_STATE_LOAD:
-            /* We using custom? */
-            if ( get_selected_theme()==get_theme_count() )
             {
-                sprintf(get_white_name(),"White");
-                sprintf(get_black_name(),"Black");
-                load_theme(get_stylelist(get_cur_style()),
-                           get_pieces_list(get_pieces_list_cur()),
-                           get_board_list(get_board_list_cur()));
-            }
-            else
-            {
-                /* printf( "Loading theme %i\n", selected_theme ); */
-                load_theme(get_theme(get_selected_theme())->style, get_theme(get_selected_theme())->pieces,
-                           get_theme(get_selected_theme())->board);
-                sprintf(get_white_name(),"%s",get_theme(get_selected_theme())->white_name);
-                sprintf(get_black_name(),"%s",get_theme(get_selected_theme())->black_name);
+                option_t *option = config_get_option("theme");
+                struct theme_struct *theme = option->selected->data;
+
+                load_theme(theme);
+                sprintf(get_white_name(),"%s", theme->white_name);
+                sprintf(get_black_name(),"%s", theme->black_name);
             }
 
             reset_3d();
@@ -392,8 +379,7 @@ static config_t *do_menu(int *pgn)
         /* Draw mouse cursor.. */
 #ifndef _arch_dreamcast
 #ifndef __BEOS__
-        /*SDL_GetMouseState(&mouse_x, &mouse_y);*/
-        draw_texture( get_mouse_cursor(), get_mouse_x(), (479-get_mouse_y()-32), 32, 32, 1.0f,
+        draw_texture( get_menu_mouse_cursor(), get_mouse_x(), (479-get_mouse_y()-32), 32, 32, 1.0f,
                       get_col(COL_WHITE) );
 #endif /* __BEOS __ */
 #endif /* _arch_dreamcast */
@@ -402,26 +388,118 @@ static config_t *do_menu(int *pgn)
     }
 }
 
-/** Implements ui_driver::init. */
-static void init_gui( int width, int height, int fullscreen)
+static void free_menu_tex()
+{
+    glDeleteTextures(1, &menu_title_tex.id);
+    glDeleteTextures(1, &get_menu_mouse_cursor()->id);
+    glDeleteTextures(1, &get_menu_border()->id);
+    glDeleteTextures(1, &get_text_character(0)->id);
+}
+
+static void load_menu_tex()
+{
+    ch_datadir();
+
+    /* For the menu.. */
+    load_texture_png( &menu_title_tex, "menu_title.png" , 0, 1);
+
+    /* New text stuff. */
+    generate_text_chars();
+
+    chdir("styles");
+    chdir("default");
+    load_border(get_menu_border(), "border.png");
+#ifndef _arch_dreamcast
+#ifndef __BEOS__
+    load_texture_png( get_menu_mouse_cursor(), "mouse_cursor.png", 1, 1 );
+#endif /* __BEOS__ */    
+#endif /* _arch_dreamcast */
+}
+
+static int set_video( int width, int height, int fullscreen, int ms )
 {
     int video_flags;
-    SDL_Surface *icon, *surface;
-    const SDL_VideoInfo *video_info;
+    SDL_Surface *surface;
+
+    video_flags = SDL_OPENGL;          /* Enable OpenGL in SDL */
+
+    DBG_LOG("setting video mode to %ix%i; fullscreen %s; %ix multisampling",
+            width, height, fullscreen ? "on" : "off", ms);
+
+    if ( fullscreen )
+        video_flags |= SDL_FULLSCREEN;
+
+    SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );
+    SDL_GL_SetAttribute( SDL_GL_STENCIL_SIZE, 1 );
+
+    if (ms) {
+        SDL_GL_SetAttribute( SDL_GL_MULTISAMPLEBUFFERS, 1 );
+    } else {
+        SDL_GL_SetAttribute( SDL_GL_MULTISAMPLEBUFFERS, 0 );
+    }
+    SDL_GL_SetAttribute( SDL_GL_MULTISAMPLESAMPLES, ms );
+
+    surface = SDL_SetVideoMode( width, height, SCREEN_BPP, video_flags );
+    if ( !surface )
+    {
+        DBG_WARN("failed to set video mode: %ix%i; fullscreen %s; %ix multisampling: %s",
+            width, height, fullscreen ? "on" : "off", ms, SDL_GetError());
+        return 1;
+    }
+
+    return 0;
+}
+
+static int resize(int width, int height, int fullscreen, int ms)
+{
+#ifdef _WIN32
+    free_menu_tex();
+#endif
+
+    if (!set_video(width, height, fullscreen, ms))
+    {
+#ifdef _WIN32
+        /* We lose the GL context on win32 */
+        init_gl();
+        load_menu_tex();
+#endif
+        resize_window(width, height);
+        screen_width=width;
+        screen_height=height;
+        screen_fs=fullscreen;
+        screen_ms=ms;
+        return 0;
+    }
+
+    /* Have we now lost our original surface? */
+    if (set_video(screen_width, screen_height, screen_fs, screen_ms))
+    {
+        DBG_ERROR("failed to restore original video mode: %ix%i; %s; %ix multisampling",
+            screen_width, screen_height, screen_fs ? "full screen" : "", screen_ms);
+        exit(1);
+    }
+
+#ifdef _WIN32
+    load_menu_tex();
+#endif
+
+    return 1;
+}
+
+/** Implements ui_driver::init. */
+static int init_gui( int width, int height, int fullscreen, int ms)
+{
+    SDL_Surface *icon;
     int i;
-    DIR* styledir;
-    struct dirent* styledir_entry;
-    char temp[80];
 
     screen_width=width;
     screen_height=height;
-
-    DBG_LOG( "screen set to %ix%i", width, height );
+    screen_fs=fullscreen;
+    screen_ms=ms;
 
     if ( SDL_Init( SDL_INIT_VIDEO | SDL_INIT_JOYSTICK | SDL_INIT_NOPARACHUTE ) < 0 )
     {
-        fprintf( stderr, "Video initialization failed: %s\n",
-                 SDL_GetError( ) );
+        DBG_ERROR("SDL initialization failed: %s", SDL_GetError());
         exit(1);
     }
 
@@ -433,7 +511,7 @@ static void init_gui( int width, int height, int fullscreen)
 
     if (!icon)
     {
-        fprintf(stderr, "Could not load icon: %s\n", SDL_GetError());
+        DBG_ERROR("failed to load icon: %s", SDL_GetError());
         exit(1);
     }
 
@@ -444,132 +522,28 @@ static void init_gui( int width, int height, int fullscreen)
     dc_draw_vmu_icon();
 #endif
 
-    video_info = SDL_GetVideoInfo( );
-
-    if ( !video_info )
+    if (set_video( screen_width, screen_height, fullscreen, ms ))
     {
-        fprintf( stderr, "Video query failed: %s\n",
-                 SDL_GetError( ) );
-        exit(1);
+        mode_set_failed = 1;
+        return 1;
     }
 
-    video_flags  = SDL_OPENGL;          /* Enable OpenGL in SDL */
-    video_flags |= SDL_GL_DOUBLEBUFFER; /* Enable double buffering */
-    video_flags |= SDL_HWPALETTE;       /* Store the palette in hardware */
-    /* video_flags |= SDL_RESIZABLE; */      /* Enable window resizing */
-    
-    if ( fullscreen )
-    {
-        video_flags |= SDL_FULLSCREEN;
-        DBG_LOG( "fullscreen set" );
-    }
+    init_gl();
+    resize_window(width, height);
 
-    if ( video_info->hw_available )
-        video_flags |= SDL_HWSURFACE;
-    else
-        video_flags |= SDL_SWSURFACE;
+    load_menu_tex();
 
-    if ( video_info->blit_hw )
-        video_flags |= SDL_HWACCEL;
-
-    SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );
-
-    surface = SDL_SetVideoMode( width, height, SCREEN_BPP, video_flags );
-    if ( !surface )
-    {
-        fprintf( stderr,  "Video mode set failed: %s\n", SDL_GetError( ) );
-        exit(1);
-    }
-
-	#ifndef __BEOS__
+#ifndef __BEOS__
     SDL_ShowCursor(SDL_DISABLE);
-	#endif /* __BEOS__ */
+#endif /* __BEOS__ */
 
     if( SDL_NumJoysticks()>0 )
         joy=SDL_JoystickOpen(0);
 
     SDL_WM_SetCaption( "DreamChess", NULL );
-    init_gl();
 
     gg_system_init(get_gg_driver_sdlgl());
 
-    /* New text stuff. */
-    generate_text_chars();
-
-    /* For the menu.. */
-    load_texture_png( &menu_title_tex, "menu_title.png" , 0, 1);
-
-    /* Load themes xml */
-
-    ch_datadir();
-    if ( (styledir=opendir("themes")) != NULL )
-    {
-        set_theme_count(0);
-        /*styledir_entry=readdir(styledir);*/
-        while ((styledir_entry = readdir(styledir)) != NULL)
-        {
-            if ( styledir_entry->d_name[0] != '.' )
-            {
-                sprintf( temp, "themes/%s", styledir_entry->d_name );
-                load_theme_xml( temp );
-            }
-        }
-        closedir(styledir);
-    }
-
-    /* Fill theme list. */
-    themelist = malloc(sizeof(char *) * (get_theme_count() + 1));
-    if ( get_theme_count() > 0 )
-    {
-        for ( i=0; i<get_theme_count(); i++ )
-            themelist[i]=strdup( get_theme(i)->name );
-    }
-    themelist[get_theme_count()]=strdup( "Custom" );
-
-    /* Fill style list. */
-    if ( (styledir=opendir("styles")) != NULL )
-    {
-        num_style = 0;
-        while ((styledir_entry = readdir(styledir)) != NULL)
-        {
-            if ( styledir_entry->d_name[0] != '.' )
-            {
-                stylelist = realloc(stylelist, (num_style + 1) *
-                                    sizeof(char *));
-                stylelist[num_style++]=strdup( styledir_entry->d_name );
-            }
-        }
-        closedir(styledir);
-    }
-
-    chdir("styles");
-    chdir("default");
-    load_border(get_menu_border(), "border.png");
-#ifndef _arch_dreamcast
-#ifndef __BEOS__
-    load_texture_png( get_mouse_cursor(), "mouse_cursor.png", 1, 1 );
-#endif /* __BEOS__ */    
-#endif /* _arch_dreamcast */
-
-    /* Fill pieces list. */
-    ch_datadir();
-
-    if ((styledir=opendir("pieces")) != NULL )
-    {
-        pieces_list_total = 0;
-        while ((styledir_entry = readdir(styledir)) != NULL)
-        {
-            if (styledir_entry->d_name[0] != '.')
-            {
-                pieces_list = realloc(pieces_list, (pieces_list_total + 1) *
-                                      sizeof(char *));
-                pieces_list[pieces_list_total++] =
-                    strdup(styledir_entry->d_name);
-            }
-        }
-        closedir(styledir);
-    }
-
     style_ingame.fade_col = gg_colour(0.0f, 0.0f, 0.0f, 0.5f);
     style_ingame.hor_pad = 20;
     style_ingame.vert_pad = 10;
@@ -585,28 +559,18 @@ static void init_gui( int width, int height, int fullscreen)
     for (i = 0; i < 9; i++)
         style_menu.border.image[i] = &get_menu_border()[i];
 
-    /* Fill board list. */
     ch_datadir();
+    audio_init();
 
     /* Make virtual keyboard table? */
     populate_key_table();
 
-    if ((styledir=opendir("boards")) != NULL )
-    {
-        board_list_total = 0;
-        while ((styledir_entry = readdir(styledir)) != NULL)
-        {
-            if (styledir_entry->d_name[0] != '.')
-            {
-                board_list = realloc(board_list, (board_list_total + 1) *
-                                     sizeof(char *));
-                board_list[board_list_total++] =
-                    strdup(styledir_entry->d_name);
-            }
-        }
-        closedir(styledir);
-    }
     update_fps_time();
+
+    /* Register music callback */
+    audio_set_music_callback(music_callback);
+
+    return 0;
 }
 
 /** Implements ui_driver::update. */
@@ -617,6 +581,13 @@ static void update(board_t *b, move_t *move)
     if ( move != NULL )
         start_piece_move( move->source, move->destination );
 
+    if ( board.state == BOARD_STALEMATE )
+    {
+        game_in_stalemate=TRUE;
+    }
+    else
+        game_in_stalemate=FALSE;
+
     if ( board.state == BOARD_CHECK )
     {
         if (IS_WHITE(board.turn))
@@ -630,6 +601,19 @@ static void update(board_t *b, move_t *move)
         white_in_check=FALSE;
     }
 
+    if ( board.state == BOARD_CHECKMATE )
+    {
+        if (IS_WHITE(board.turn))
+            white_in_checkmate=TRUE;
+        else
+            black_in_checkmate=TRUE;
+    }
+    else
+    {
+        black_in_checkmate=FALSE;
+        white_in_checkmate=FALSE;
+    }
+
     /* FIXME */
     while ( get_piece_moving_done() == 0 )
         poll_move();
@@ -642,51 +626,22 @@ static void show_result(result_t *res)
 }
 
 /** Implements ui_driver::init. */
-static int sdlgl_init(int width, int height, int fullscreen)
+static int sdlgl_init(int width, int height, int fullscreen, int ms)
 {
-    init_gui(width,height,fullscreen);
-    return 0;
+    return init_gui(width,height,fullscreen,ms);
 }
 
 /** Implements ui_driver::exit. */
 static int sdlgl_exit()
 {
-    int i;
+    audio_set_music_callback(NULL);
+
     gg_system_exit();
-    glDeleteTextures(1, &menu_title_tex.id);
+    free_menu_tex();
 
     if (joy)
         SDL_JoystickClose(joy);
 
-    if (themelist)
-    {
-        for (i = 0; i < get_theme_count() + 1; i++)
-            free(themelist[i]);
-        free(themelist);
-    }
-
-    if (stylelist)
-    {
-        for (i = 0; i < num_style; i++)
-            free(stylelist[i]);
-        free(stylelist);
-    }
-
-    if (pieces_list)
-    {
-        for (i = 0; i < pieces_list_total; i++)
-            free(pieces_list[i]);
-
-        free(pieces_list);
-    }
-
-    if (board_list)
-    {
-        for (i = 0; i < board_list_total; i++)
-            free(board_list[i]);
-        free(board_list);
-    }
-
     SDL_Quit();
     return 0;
 }
@@ -705,7 +660,8 @@ static void poll_move()
     move_t move;
     int input;
 
-    draw_scene(&board);
+    audio_poll(0);
+    draw_scene(&board, reflections);
 
     if (switch_to_menu == TRUE)
     {
@@ -729,6 +685,14 @@ static void poll_move()
 
     input = get_move();
 
+    /* FIXME */
+    if (!engine_error_shown && game_get_engine_error())
+    {
+        gg_dialog_open(dialog_engine_error_create());
+        engine_error_shown = 1;
+        return;
+    }
+
     if (!game_want_move())
     {
         source = -1;
@@ -799,6 +763,7 @@ ui_driver_t ui_sdlgl =
     {
         "sdlgl",
         sdlgl_init,
+        resize,
         sdlgl_exit,
         do_menu,
         update,
