@@ -1,20 +1,22 @@
 /*  DreamChess
- *  Copyright (C) 2005  The DreamChess project
- *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
+**
+**  DreamChess is the legal property of its developers, whose names are too
+**  numerous to list here. Please refer to the COPYRIGHT file distributed
+**  with this source distribution.
+**
+**  This program is free software: you can redistribute it and/or modify
+**  it under the terms of the GNU General Public License as published by
+**  the Free Software Foundation, either version 3 of the License, or
+**  (at your option) any later version.
+**
+**  This program is distributed in the hope that it will be useful,
+**  but WITHOUT ANY WARRANTY; without even the implied warranty of
+**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+**  GNU General Public License for more details.
+**
+**  You should have received a copy of the GNU General Public License
+**  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*/
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -30,24 +32,13 @@
 #define TRUE  1
 #define FALSE 0
 
-#ifdef _arch_dreamcast
-#include <dc/fmath.h>
-#define sin fsin
-#define cos fcos
-#define sqrt fsqrt
-#define magnitude_sqr(X, Y, Z) fipr_magnitude_sqr(X, Y, Z, 0)
-#define dot_product(X, Y, Z, XX, YY, ZZ) fipr(X, Y, Z, 0, XX, YY, ZZ, 0)
-#else
-#include <math.h>
-#define magnitude_sqr in_product_self
-#define dot_product in_product
-#endif
-
 #include <SDL.h>
 #include <SDL_opengl.h>
 
 #include "board.h"
+#include "ui_sdlgl.h"
 #include "ui_sdlgl_3d.h"
+#include "theme.h"
 
 typedef struct coord3
 {
@@ -65,31 +56,6 @@ typedef struct coord3d
 }
 coord3d_t;
 
-typedef struct texture
-{
-    /** OpenGL Texture ID. */
-    GLuint id;
-
-    /** Lower-left u-coordinate. Ranges from 0.0f to 1.0f. */
-    float u1;
-
-    /** Lower-left v-coordinate. Ranges from 0.0f to 1.0f. */
-    float v1;
-
-    /** Upper-right u-coordinate. Ranges from 0.0f to 1.0f. */
-    float u2;
-
-    /** Upper-right v-coordinate. Ranges from 0.0f to 1.0f. */
-    float v2;
-
-    /** Width of texture in pixels. */
-    int width;
-
-    /** Height of texture in pixels. */
-    int height;
-}
-texture_t;
-
 typedef enum primitive_type
 {
     PRIM_TRIANGLES,
@@ -100,17 +66,31 @@ typedef struct group
 {
     primitive_type_t type;
     int len;
-    unsigned int *data;
+    GLuint *data;
 }
 group_t;
 
+typedef struct bone
+{
+    char *name;
+    float offset[3];
+    int children;
+    int *child;
+}
+bone_t;
+
 typedef struct mesh
 {
-    float *vertex;
-    float *normal;
-    float *tex_coord;
+    int has_bones;
+    int vertices;
+    GLfloat *vertex;
+    GLfloat *normal;
+    GLfloat *tex_coord;
+    int *bone_w;
     int groups;
     group_t *group;
+    bone_t *bone;
+    GLuint list;
 }
 mesh_t;
 
@@ -178,7 +158,10 @@ static void data_col_add(data_col_t *data_col, char *name, void *data)
 static data_col_t textures;
 static data_col_t meshes;
 
-texture_t ground;
+#define SEL_BOUNCE_AMP 0.25f
+#define SEL_HEIGHT 0.003f
+static theme_selector_t sel;
+static texture_t sel_tex;
 
 #define BUF_SIZE 256
 #define FN_LEN 256
@@ -194,26 +177,36 @@ static inline float in_product(float x, float y, float z, float xx, float yy,
     return x * xx + y * yy + z * zz;
 }
 
-static model_t model[12];
+static model_t model[13];
 static model_t board;
 
 static int is_2d;
 
+#define SELECTOR_SHOW_TICKS 5000
 static int selector, selected;
 static float x_rotation, z_rotation;
+static int selector_hide_time;
 
-int piece_moving_done=1;
-int piece_moving_start;
-int piece_moving_dest;
-int piece_moving_source;
-int piece_moving_x_done;
-int piece_moving_y_done;
-float piece_moving_source_xpos;
-float piece_moving_source_ypos;
-float piece_moving_dest_xpos;
-float piece_moving_dest_ypos;
-float piece_moving_xpos;
-float piece_moving_ypos;
+static int piece_moving_done=1;
+static int piece_moving_start;
+static int piece_moving_dest;
+static int piece_moving_source;
+static int piece_moving_x_done;
+static int piece_moving_y_done;
+static float piece_moving_source_xpos;
+static float piece_moving_source_ypos;
+static float piece_moving_dest_xpos;
+static float piece_moving_dest_ypos;
+static float piece_moving_xpos;
+static float piece_moving_ypos;
+
+static void setup_view()
+{
+    glLoadIdentity();
+    glTranslatef(0, -0.5f, -12.0f);
+    glRotatef(x_rotation, 1, 0, 0);
+    glRotatef(z_rotation, 0, 0, 1);
+}
 
 int get_piece_moving_done()
 {
@@ -297,7 +290,9 @@ mesh_t *dcm_load(char *filename)
 
     mesh = malloc(sizeof(mesh_t));
 
-    mesh->vertex = malloc(sizeof(float) * vertices * 3);
+    mesh->has_bones = 0;
+    mesh->vertices = vertices;
+    mesh->vertex = malloc(sizeof(GLfloat) * vertices * 3);
 
     for (i = 0; i < vertices * 3; i++)
     {
@@ -308,7 +303,7 @@ mesh_t *dcm_load(char *filename)
         }
     }
 
-    mesh->normal = malloc(sizeof(float) * vertices * 3);
+    mesh->normal = malloc(sizeof(GLfloat) * vertices * 3);
 
     for (i = 0; i < vertices * 3; i++)
     {
@@ -319,7 +314,7 @@ mesh_t *dcm_load(char *filename)
         }
     }
 
-    mesh->tex_coord = malloc(sizeof(float) * vertices * 2);
+    mesh->tex_coord = malloc(sizeof(GLfloat) * vertices * 2);
 
     for (i = 0; i < vertices * 2; i++)
     {
@@ -368,7 +363,7 @@ mesh_t *dcm_load(char *filename)
 
         mesh->group[i].len = group_len;
 
-        mesh->group[i].data = malloc(sizeof(unsigned int) * group_len);
+        mesh->group[i].data = malloc(sizeof(GLuint) * group_len);
 
         for (j = 0; j < group_len; j++)
         {
@@ -385,6 +380,174 @@ mesh_t *dcm_load(char *filename)
     return mesh;
 }
 
+#if 0
+mesh_t *dcm_load_new(char *filename)
+{
+    FILE *f;
+    mesh_t *mesh;
+    int version;
+    char id[3];
+    int vertices;
+    int bones;
+    int i;
+
+    f = fopen(filename, "r");
+
+    if (!f)
+    {
+        DBG_ERROR("couldn't open %s", filename);
+        return NULL;
+    }
+
+    if ((fscanf(f, "%c%c%c %d\n", &id[0], &id[1], &id[2], &version) != 4)
+            || ((id[0] != 'D') || (id[1] != 'C') || (id[2] != 'M')))
+    {
+        DBG_ERROR("invalid DCM file header");
+        return NULL;
+    }
+
+    if (version != 1)
+    {
+        DBG_ERROR( "DCM version %i not supported", version);
+        return NULL;
+    }
+
+    if (fscanf(f, "VERTICES %d\n", &vertices) != 1)
+    {
+        DBG_ERROR("error reading DCM file");
+        return NULL;
+    }
+
+    mesh = malloc(sizeof(mesh_t));
+    mesh->has_bones = 1;
+
+    mesh->vertex = malloc(sizeof(float) * vertices * 3);
+    mesh->normal = malloc(sizeof(float) * vertices * 3);
+
+    for (i = 0; i < vertices; i++)
+    {
+        if (fscanf(f, "%f %f %f %f %f %f\n", &mesh->vertex[i * 3], &mesh->vertex[i * 3 + 1], &mesh->vertex[i * 3 + 2],
+            &mesh->normal[i * 3], &mesh->normal[i * 3 + 1], &mesh->normal[i * 3 + 2]) != 6)
+        {
+            DBG_ERROR("error reading DCM file: %i of %i", i, vertices);
+            exit(1);
+        }
+    }
+
+    mesh->bone_w = malloc(sizeof(int) * vertices);
+    memset(mesh->bone_w, -1, sizeof(int) * vertices);
+
+    mesh->tex_coord = malloc(sizeof(float) * vertices * 2);
+
+    for (i = 0; i < vertices * 2; i++)
+        mesh->tex_coord[i] = 0;
+
+    /* As we don't flip our images we flip our u coordinates instead. */
+    for (i = 1; i < vertices * 2; i += 2)
+        mesh->tex_coord[i] = 1.0f - mesh->tex_coord[i];
+
+    mesh->groups = 1;
+    mesh->group = malloc(sizeof(group_t) * mesh->groups);
+
+    for (i = 0; i < mesh->groups; i++)
+    {
+        int group_len;
+        float fl;
+        int j;
+
+        mesh->group[i].type = PRIM_TRIANGLES;
+
+        if (fscanf(f, "TRIANGLES %d\n", &group_len) != 1)
+        {
+            DBG_ERROR("error reading DCM file");
+            exit(1);
+        }
+
+        mesh->group[i].len = group_len * 3;
+
+        mesh->group[i].data = malloc(sizeof(unsigned int) * group_len * 3);
+
+        for (j = 0; j < group_len; j++)
+        {
+            if (fscanf(f, "%u %u %u %f %f %f %f %f %f\n", &mesh->group[i].data[j * 3], &mesh->group[i].data[j * 3 + 1],
+                &mesh->group[i].data[j * 3 + 2], &fl, &fl, &fl, &fl, &fl, &fl) != 9)
+            {
+                DBG_ERROR("error reading DCM file");
+                exit(1);
+            }
+        }
+    }
+
+    if (fscanf(f, "BONES %d\n", &bones) != 1)
+    {
+        DBG_ERROR("error reading DCM file");
+        return NULL;
+    }
+
+    mesh->bone = malloc(sizeof(bone_t) * bones);
+
+    for (i = 0; i < bones; i++)
+    {
+        int j;
+        int vw;
+
+        mesh->bone[i].name = malloc(256);
+        if (fscanf(f, "BONE %255s\n", mesh->bone[i].name) != 1)
+        {
+            DBG_ERROR("error reading DCM file");
+            exit(1);
+        }
+
+        if (fscanf(f, "%f %f %f\n", &mesh->bone[i].offset[0], &mesh->bone[i].offset[1],
+            &mesh->bone[i].offset[2]) != 3)
+        {
+            DBG_ERROR("error reading DCM file");
+            exit(1);
+        }
+
+        if (fscanf(f, "CHILDREN %d\n", &mesh->bone[i].children) != 1)
+        {
+            DBG_ERROR("error reading DCM file");
+            exit(1);
+        }
+
+        if (mesh->bone[i].children)
+            mesh->bone[i].child = malloc(sizeof(int) * mesh->bone[i].children);
+
+        for (j = 0; j < mesh->bone[i].children; j++)
+        {
+            if (fscanf(f, "%d\n", &mesh->bone[i].child[j]) != 1)
+            {
+                DBG_ERROR("error reading DCM file");
+                exit(1);
+            }
+        }
+
+        if (fscanf(f, "VERTEXWEIGHTS %d\n", &vw) != 1)
+        {
+            DBG_ERROR("error reading DCM file");
+            exit(1);
+        }
+
+        for (j = 0; j < vw; j++)
+        {
+            float weight;
+            int vert_idx;
+            if (fscanf(f, "%i %f\n", &vert_idx, &weight) != 2)
+            {
+                DBG_ERROR("error reading DCM file");
+                exit(1);
+            }
+            mesh->bone_w[vert_idx] = i;
+        }
+    }
+
+    fclose(f);
+
+    return mesh;
+}
+#endif
+
 static mesh_t *load_mesh(char *filename)
 {
     mesh_t *mesh = data_col_find(&meshes, filename);
@@ -400,34 +563,54 @@ static mesh_t *load_mesh(char *filename)
     return mesh;
 }
 
-coord3_t light = {0.0f, 0.0f, -1.0f};
-
-#define DC_PI 3.14159265358979323846
-
-inline float arccos(float f)
+#if 0
+static mesh_t *load_mesh_new(char *filename)
 {
-    return (2.193376378 + (-2.987042783 + (.5314426631 + .2990387380 * f)
-                           * f) * f) / (1.396346817 + (-1.012703522 + (-.3056194995
-                                                       + .1383216735 * f) * f) * f);
+    mesh_t *mesh = data_col_find(&meshes, filename);
+
+    if (mesh)
+    {
+        return mesh;
+    }
+
+    DBG_LOG("loading mesh: %s", filename);
+    mesh = dcm_load_new(filename);
+    data_col_add(&meshes, filename, mesh);
+    return mesh;
 }
+#endif
 
-void model_render(model_t *model, float alpha, coord3_t *light, char tex_spin )
+void model_render(model_t *model, int specular, float alpha)
 {
-    mesh_t *mesh = model->mesh;
-    int g;
+    float specReflection[] = { 1.0f, 1.0f, 1.0f, 1.0f };
+    float nospecReflection[] = { 0.0f, 0.0f, 0.0f, 1.0f };
+    float mcolor[4] = { 1.0f, 1.0f, 1.0f };
     texture_t *texture = model->texture;
-    int ticks = SDL_GetTicks();
-    float tex_spin_pos=0.0f;
+
+    mcolor[3] = alpha;
+    glMaterialfv(GL_FRONT, GL_DIFFUSE, mcolor);
+
+    if (specular)
+        glMaterialfv(GL_FRONT, GL_SPECULAR, specReflection);
+    else
+        glMaterialfv(GL_FRONT, GL_SPECULAR, nospecReflection);
+    glMateriali(GL_FRONT, GL_SHININESS, 128);
 
     glEnable(GL_TEXTURE_2D);
     glBindTexture(GL_TEXTURE_2D, texture->id);
 
-    if ( tex_spin )
-        tex_spin_pos=(float)ticks / (float)(1000 * (1000/(float)get_tex_spin_speed()));
+    glCallList(model->mesh->list);
 
-    /* printf( "pos: %f\n", tex_spin_pos );
-    printf( "%f\n", (1000/(float)get_tex_spin_speed()) );
-    tex_spin_pos=(float)ticks / (float)(1000/(float)get_tex_spin_speed()); */
+    glDisable(GL_TEXTURE_2D);
+}
+
+void model_make_list(model_t *model)
+{
+    int g;
+    mesh_t *mesh = model->mesh;
+
+    mesh->list = glGenLists(1);
+    glNewList(mesh->list, GL_COMPILE);
 
     for (g = 0; g < mesh->groups; g++)
     {
@@ -445,42 +628,22 @@ void model_render(model_t *model, float alpha, coord3_t *light, char tex_spin )
         for (i = 0; i < mesh->group[g].len; i++)
         {
             unsigned int *data = mesh->group[g].data;
-            float angle = 1.0f;
 
-            if (light && use_lighting())
-            {
-                angle = arccos(dot_product(mesh->normal[data[i] * 3],
-                                           mesh->normal[data[i] * 3 + 1],
-                                           mesh->normal[data[i] * 3 + 2],
-                                           light->x, light->y, light->z));
-
-                angle /= 2.8;
-
-                if (angle < 0.5f)
-                    angle = 0.25f;
-
-                else
-                {
-                    angle -= 0.5f;
-                    angle *= 1.5;
-                    angle += 0.25f;
-                }
-            }
-
-            glColor4f(angle, angle, angle, alpha);
-
-            glTexCoord2f(mesh->tex_coord[data[i] * 2] * texture->u2+tex_spin_pos,
-                         mesh->tex_coord[data[i] * 2 + 1] * texture->v2);
-
-            glVertex3f(mesh->vertex[data[i] * 3],
-                       mesh->vertex[data[i] * 3 + 1],
-                       mesh->vertex[data[i] * 3 + 2]);
+            glTexCoord2fv(mesh->tex_coord + data[i] * 2);
+            glNormal3fv(mesh->normal + data[i] * 3);
+            glVertex3fv(mesh->vertex + data[i] * 3);
         }
 
         glEnd();
     }
 
-    glDisable(GL_TEXTURE_2D);
+    glEndList();
+}
+
+void set_theme(struct theme_struct *theme, texture_t texture)
+{
+    sel = theme->selector;
+    sel_tex = texture;
 }
 
 void loadmodels(char *filename)
@@ -516,8 +679,13 @@ void loadmodels(char *filename)
 
         model[i].mesh = load_mesh(mesh);
         model[i].texture = load_piece_texture(texture);
+        model_make_list(model + i);
     }
-
+/*
+    model[12].mesh = load_mesh_new("/home/walter/devel/ginger/ginger.dcm");
+    model[12].texture = load_piece_texture(texture);
+    model[11].mesh = model[12].mesh;
+*/
     if ((model[0].mesh->groups) == 1 && (model[0].mesh->group[0].len == 4))
         is_2d = 1;
     else
@@ -530,6 +698,7 @@ void load_board(char *dcm_name, char *texture_name)
 {
     board.mesh = load_mesh(dcm_name);
     board.texture = load_piece_texture(texture_name);
+    model_make_list(&board);
 }
 
 void free_mesh(void *data)
@@ -565,30 +734,26 @@ void freemodels()
 /* How many squares per second? */
 #define PIECE_MOVE_SPEED 3.0
 
-extern float piece_moving_source_xpos;
-extern float piece_moving_source_ypos;
-extern float piece_moving_dest_xpos;
-extern float piece_moving_dest_ypos;
-extern float piece_moving_xpos;
-extern float piece_moving_ypos;
-extern int piece_moving_dest;
-extern int piece_moving_source;
-extern int piece_moving_start;
-extern int piece_moving_x_done;
-extern int piece_moving_y_done;
-extern int piece_moving_done;
+/* Draw a selected piece after the rest.. */
+static int selected_piece_render;
+static int selected_piece_model;
+static int selected_piece_grab;
 
-static void draw_pieces(board_t *board, float rot_x, float rot_z)
+/* Draw any moving piece.. */
+static int moving_piece_render;
+static int moving_piece_model;
+static int moving_piece_grab;
+
+static void draw_pieces(board_t *board, int flip)
 {
     int i,j,k;
     float moved=0;
-    coord3_t light_inv;
-
-    light_inv.x = -light.x;
-    light_inv.y = -light.y;
-    light_inv.z = light.z;
-
+ 
     moved=(float)((SDL_GetTicks()-piece_moving_start)/(1000/PIECE_MOVE_SPEED));
+    selected_piece_grab=FALSE;
+    moving_piece_grab=FALSE;
+    selected_piece_render=FALSE;  
+    moving_piece_render=FALSE;  
 
     /* Draw the pieces.. */
     for (i = 7; i >= 0; i--)
@@ -596,11 +761,10 @@ static void draw_pieces(board_t *board, float rot_x, float rot_z)
         {
             if ((k = board->square[(0/*flip_board*/ ? 63 - (i*8+j) : i*8+j)]) != NONE)
             {
-                coord3_t *l = &light;
-                glLoadIdentity();
-                glTranslatef(0, -0.5f, -12.0f );
-                glRotatef(rot_x, 1, 0, 0);
-                glRotatef(rot_z, 0, 0, 1);
+                setup_view();
+
+                if (!is_2d && flip)
+                    glScalef(1.0f, 1.0f, -1.0f);
 
                 if ( (i*8+j) == piece_moving_dest )
                 {
@@ -637,13 +801,14 @@ static void draw_pieces(board_t *board, float rot_x, float rot_z)
                     }
                     if (is_2d)
                         glTranslatef(-3.5f + piece_moving_xpos, -3.5f +
-                                     piece_moving_ypos, 0.04);
+                                     piece_moving_ypos, 0.002);
                     else
                         glTranslatef(-3.5f + piece_moving_xpos, -3.5f +
-                                     piece_moving_ypos, 0.02);
+                                     piece_moving_ypos, 0.001);
+
                 }
                 else
-                    glTranslatef(-3.5f + j, -3.5f + i, 0.02);
+                    glTranslatef(-3.5f + j, -3.5f + i, 0.001);
 
                 if (is_2d)
                 {
@@ -651,71 +816,117 @@ static void draw_pieces(board_t *board, float rot_x, float rot_z)
 
                     glRotatef(-steps * 90.0f, 0, 0, 1);
                 }
-                else if (k < 12 && IS_BLACK(k))
+/*                else if (k < 12 && IS_BLACK(k))
                 {
                     glRotatef(180, 0, 0, 1);
-                    l = &light_inv;
-                }
+                }*/
 
-                if ( !is_2d && piece_moving_done == 0 &&
-                        (i*8+j) == piece_moving_dest )
+                if ( i * 8 + j == selected )
                 {
+                    selected_piece_render=TRUE;
+                    selected_piece_model=k;
+                    selected_piece_grab=TRUE;
                     glPushMatrix();
-                    /* glRotatef( 10.0f, xrot, yrot, 0.0f ); */
-                    model_render(&model[k], (i * 8 + j == selected ? 0.5f : 1.0f), l, use_tex_spin());
-                    glPopMatrix();
-
                 }
                 else
-                    model_render(&model[k], (i * 8 + j == selected ? 0.5f : 1.0f), l, use_tex_spin());
+                    selected_piece_grab=FALSE;
+
+                if ( piece_moving_done == FALSE && piece_moving_dest == (i*8+j) )
+                {
+                    moving_piece_render=TRUE;
+                    moving_piece_model=k;
+                    moving_piece_grab=TRUE;
+                    glPushMatrix();
+                }             
+                else
+                    moving_piece_grab=FALSE;          
+
+                if ( !selected_piece_grab && !moving_piece_grab )
+                    model_render(&model[k], !is_2d, (i * 8 + j == selected ? 0.5f : 1.0f));
             }
         }
+
+        /* Do we need to draw a selected piece? */
+        if ( selected_piece_render )
+        {
+            glPopMatrix();
+            model_render(&model[selected_piece_model], !is_2d, 0.5f);
+        }
+
+        if ( moving_piece_render )
+        {
+            glPopMatrix();
+            model_render(&model[moving_piece_model], !is_2d, 1.0f);
+        }
 }
 
-static void draw_board(float rot_x, float rot_z)
+static void draw_board(int blend)
 {
-    coord3_t fixed = {0, 0, -1};
-    glLoadIdentity();
-    glTranslatef(0, -0.5f, -12.0f );
-    glRotatef(rot_x, 1, 0, 0);
-    glRotatef(rot_z, 0, 0, 1);
-
-    model_render(&board, 1.0f, &fixed, FALSE);
+    setup_view();
+    model_render(&board, 0, 1.0f);
 }
 
-void draw_selector()
+void draw_selector(float alpha)
 {
-    float r,g,b,a;
-    float width;
-    float height;
+    float bounce_offset = 0.0;
+    float spin_offset = 0.0;
+    Uint32 ticks = SDL_GetTicks();
 
-    r=1.0f; g=1.0f; b=0.0f; a=0.25f; /* a=((((SDL_GetTicks() % (1000 / 1)) / (float) (1000 / 1)))/4)+0.1; */
-    width=0.5;
-    height=0.1;
+    if (sel.bouncespeed)
+    {
+        float phase = ((ticks % (int) (1000 / sel.bouncespeed)) / (float) (1000 / sel.bouncespeed));
 
-    glLoadIdentity();
-    glTranslatef(0, -0.5, -12.0);
-    glRotatef(x_rotation, 1, 0, 0);
-    glRotatef(z_rotation, 0, 0, 1);
-    glTranslatef(-3.5 + selector % 8, -3.5 + selector / 8, 0.01f);
-    glColor4f(r, g, b, a);
+        if (phase < 0.5)
+            bounce_offset = phase * 2 * SEL_BOUNCE_AMP;
+        else
+            bounce_offset = (1.0 - phase) * 2 * SEL_BOUNCE_AMP;
+    }
+
+    if (sel.spinspeed)
+    {
+        float phase = ((ticks % (int) (1000 / sel.spinspeed)) / (float) (1000 / sel.spinspeed));
+
+        spin_offset = phase * 360;
+    }
+
+    setup_view();
+
+    glTranslatef(-3.5 + selector % 8, -3.5 + selector / 8, 0);
+    glRotatef(spin_offset, 0, 0, 1);
+
+    glColor4f(0.0f, 0.0f, 0.0f, 1.0f * alpha);
+
+    glEnable(GL_TEXTURE_2D);
+    glBindTexture(GL_TEXTURE_2D, sel_tex.id);
 
     glBegin(GL_QUADS);
-    glVertex3f(-width, width, height);
-    glVertex3f(width, width, height);
-    glVertex3f(width, -width, height);
-    glVertex3f(-width, -width, height);
+    glTexCoord2f(0,0);
+    glVertex3f(-sel.size/2, sel.size/2, SEL_HEIGHT);
+    glTexCoord2f(1,0);
+    glVertex3f(sel.size/2, sel.size/2, SEL_HEIGHT);
+    glTexCoord2f(1,1);
+    glVertex3f(sel.size/2, -sel.size/2, SEL_HEIGHT);
+    glTexCoord2f(0,1);
+    glVertex3f(-sel.size/2, -sel.size/2, SEL_HEIGHT);
     glEnd();
 
+    glTranslatef(0, 0, bounce_offset);
 
+    glColor4f(sel.colour[0], sel.colour[1], sel.colour[2], sel.colour[3] * alpha);
+    glBegin(GL_QUADS);
+    glTexCoord2f(0,0);
+    glVertex3f(-sel.size/2, sel.size/2, SEL_HEIGHT);
+    glTexCoord2f(1,0);
+    glVertex3f(sel.size/2, sel.size/2, SEL_HEIGHT);
+    glTexCoord2f(1,1);
+    glVertex3f(sel.size/2, -sel.size/2, SEL_HEIGHT);
+    glTexCoord2f(0,1);
+    glVertex3f(-sel.size/2, -sel.size/2, SEL_HEIGHT);
+    glEnd();
+
+    glDisable(GL_TEXTURE_2D);
 }
 
-#ifdef _arch_dreamcast
-int find_square(int x, int y, float fd)
-{
-    return -1;
-}
-#else
 int find_square(int x, int y, float fd)
 {
     coord3d_t obj;
@@ -723,11 +934,8 @@ int find_square(int x, int y, float fd)
     GLint viewport[4];
     GLdouble modelview[16];
     GLdouble projection[16];
-    
-    glLoadIdentity();
-    glTranslatef(0, -0.5f, -12.0f );
-    glRotatef(x_rotation, 1, 0, 0);
-    glRotatef(z_rotation, 0, 0, 1);
+
+    setup_view();
 
     glGetDoublev(GL_MODELVIEW_MATRIX, modelview);
     glGetDoublev(GL_PROJECTION_MATRIX, projection);
@@ -747,30 +955,105 @@ int find_square(int x, int y, float fd)
 
     return (floor(obj.y) + 4) * 8 + floor(obj.x) + 4;
 }
-#endif
 
-void render_scene_3d(board_t *board)
+static void draw_board_center(float r, float g, float b, float a)
 {
-    glEnable(GL_CULL_FACE);
-    draw_board(x_rotation, z_rotation);
-    draw_pieces(board, x_rotation, z_rotation);
-    glDisable(GL_CULL_FACE);
-    draw_selector();
+    float tc = 46 / 512.0f;
+    float mcolor[4] = { 1.0f, 1.0f, 1.0f };
+
+    mcolor[3] = a;
+    glMaterialfv(GL_FRONT, GL_DIFFUSE, mcolor);
+
+    setup_view();
+
+    glEnable(GL_TEXTURE_2D);
+    glBindTexture(GL_TEXTURE_2D, board.texture->id);
+
+    glBegin(GL_QUADS);
+    glColor4f(r, g, b, a);
+    glTexCoord2f(tc, 1 - tc);
+    glNormal3f(0, 0, 1);
+    glVertex3f(-4, -4, 0);
+    glTexCoord2f(1 - tc, 1 - tc);
+    glNormal3f(0, 0, 1);
+    glVertex3f(4, -4, 0);
+    glTexCoord2f(1 - tc, tc);
+    glNormal3f(0, 0, 1);
+    glVertex3f(4, 4, 0);
+    glTexCoord2f(tc, tc);
+    glNormal3f(0, 0, 1);
+    glVertex3f(-4, 4, 0);
+    glEnd();
+
+    glDisable(GL_TEXTURE_2D);
 }
 
-static void update_light()
+static void setup_stencil()
 {
-    float len;
+    float tc = 46 / 512.0f;
 
-    light.x = sin(z_rotation * DC_PI / 180.0f) * (1.0f - cos(x_rotation * DC_PI / 180.0f));
-    light.y = -sin(x_rotation * DC_PI / 180.0f) * cos(z_rotation * DC_PI / 180.0f);
-    light.z = -cos(x_rotation * DC_PI / 180.0f);
+    glDisable(GL_LIGHTING);
+    glClearStencil(0);
+    glClear(GL_STENCIL_BUFFER_BIT);
+    glDepthMask(GL_FALSE);
+    glDisable(GL_DEPTH_TEST);
+    glStencilFunc(GL_ALWAYS, 1, 1);
+    glStencilOp(GL_REPLACE, GL_REPLACE, GL_REPLACE);
+    glEnable(GL_STENCIL_TEST);
 
-    len = sqrt(magnitude_sqr(light.x, light.y, light.z));
+    setup_view();
 
-    light.x /= len;
-    light.y /= len;
-    light.z /= len;
+    glEnable(GL_TEXTURE_2D);
+    glBindTexture(GL_TEXTURE_2D, board.texture->id);
+
+    glBegin(GL_QUADS);
+    glColor4f(0, 0, 0, 1);
+    glTexCoord2f(tc, tc);
+    glVertex3f(-4, -4, 0);
+    glTexCoord2f(1 - tc, tc);
+    glVertex3f(4, -4, 0);
+    glTexCoord2f(1 - tc, 1 - tc);
+    glVertex3f(4, 4, 0);
+    glTexCoord2f(tc, 1 - tc);
+    glVertex3f(-4, 4, 0);
+    glEnd();
+
+    glDisable(GL_TEXTURE_2D);
+    glEnable(GL_DEPTH_TEST);
+    glStencilFunc(GL_EQUAL, 1, 1);
+    glStencilOp(GL_KEEP, GL_KEEP, GL_KEEP);
+    glDepthMask(GL_TRUE);
+    glEnable(GL_LIGHTING);
+}
+
+void render_scene_3d(board_t *board, int reflections)
+{
+  int ticks = SDL_GetTicks();
+
+    glEnable(GL_LIGHTING);
+
+    glEnable(GL_CULL_FACE);
+    if (!is_2d && reflections) {
+        setup_stencil();
+        glCullFace(GL_FRONT);
+        draw_pieces(board, 1);
+        glDisable(GL_STENCIL_TEST);
+        glCullFace(GL_BACK);
+        draw_board_center(1.0f, 1.0f, 1.0f, 0.8f);
+        draw_board(0);
+        draw_pieces(board, 0);
+    } else {
+        draw_board_center(0.75f, 0.75f, 0.75f, 1.0f);
+        draw_board(0);
+        draw_pieces(board, 0);
+    }
+    glDisable(GL_CULL_FACE);
+    glDisable(GL_LIGHTING);
+
+    if (ticks <= selector_hide_time)
+        draw_selector(1.0f);
+    else if (ticks < selector_hide_time + 1000)
+        draw_selector(1.0f - ((ticks - selector_hide_time) / 1000.0f));
 }
 
 void move_camera(float x, float z)
@@ -788,14 +1071,14 @@ void move_camera(float x, float z)
 
     if (z_rotation <= 0.0f)
         z_rotation = 360.0f;
-
-    update_light();
 }
 
 void move_selector(int direction)
 {
     int steps = (z_rotation + 45.0f) / 90.0f;
 
+    selector_hide_time = SDL_GetTicks() + SELECTOR_SHOW_TICKS;
+
     while (steps--)
     {
         switch (direction)
@@ -842,26 +1125,50 @@ int get_selector()
 void select_piece(int square)
 {
     selected = square;
-
-    if ( square>=0 )
-        selector = square;
 }
 
 void reset_3d()
 {
+    float mcolor[] = { 1.0f, 1.0f, 1.0f };
+    float position[] = { 50.0f, 10.0f, 50.0f, 1.0f };
+
     selected = -1;
     selector = 0;
+    selector_hide_time = 0;
+
+    glLightfv(GL_LIGHT0, GL_POSITION, position);
+
     if (is_2d)
     {
-        x_rotation = -10.0f;
+	GLfloat ambientLight[] = { 1.0f, 1.0f, 1.0f, 1.0f };
+
+        x_rotation = 0.0f;
         z_rotation = 0.0f;
+
+	/* Create light components */
+	glLightfv(GL_LIGHT0, GL_AMBIENT, ambientLight);
     }
     else
     {
-        x_rotation = -45.0f;
+	/* Create light components */
+	GLfloat ambientLight[] = { 0.15f, 0.15f, 0.15f, 1.0f };
+	GLfloat diffuseLight[] = { 0.45f, 0.45f, 0.45f, 1.0f };
+	GLfloat specularLight[] = { 1.0f, 1.0f, 1.0f, 1.0f };
+
+        x_rotation = -30.0f;
         z_rotation = 0.0f;
+	  	 
+	/* Assign created components to GL_LIGHT0 */
+	glLightfv(GL_LIGHT0, GL_AMBIENT, ambientLight);
+	glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuseLight);
+	glLightfv(GL_LIGHT0, GL_SPECULAR, specularLight);
     }
-    update_light();
+
+    /* White specular highlights */
+    glLightModelf(GL_LIGHT_MODEL_COLOR_CONTROL, GL_SEPARATE_SPECULAR_COLOR);
+    glMaterialfv(GL_FRONT, GL_AMBIENT, mcolor);
+
+    glEnable(GL_LIGHT0);
 }
 
 #endif /* WITH_UI_SDLGL */
